--- ../1.1.3/ugtest/ugtest.c	Fri Aug 10 14:54:17 2001
+++ ugtest/ugtest.c	Thu Oct 11 22:29:27 2001
@@ -1,71 +1,200 @@
 #include <stdio.h>
 #include <string.h>
 #include <strings.h>
+#include <sl.h>
+
 
 #include "ug1.h"
 #include "ssha.h"
+#include "xattrs.h"
 
-#define CONFIG_FILE "ug.conf"
+#define CONFIG_FILE "/etc/ug.conf"
 
-int
-main (int argc, char *argv[])
-{
     int d;
-    uint32_t *r;
-    uint32_t *cp;
-    char **v;
-    int i;
-    uint32_t uid, gid;
-    uint32_t client;
 
+struct attributes {
+    char *name;
+    uint32_t value;
+};
+
+struct attributes attrlist[] = {
+	{ "uname", XUATTR_UNAME },
+	{ "kthid", XUATTR_UNAME },
+	{ "pnr", XUATTR_PERSONNUMMER },
+	{ "personnummer", XUATTR_PERSONNUMMER },
+	{ "handle", XUATTR_HANDLE },
+	{ "ename", XUATTR_ENAME },
+	{ "family-name", XUATTR_FAMILY },
+	{ "given-name", XUATTR_GIVEN },
+	{ "password", XUATTR_PASSWORD },
+	{ "mail-forward", XUATTR_MAIL_FORWARD },
+	{ "uid", XUATTR_UID },
+	{ "ssha-hash", XUATTR_SSHA_HASH },
+	{ "mail-quota", XUATTR_MAIL_QUOTA },
+	{ "operator-ssha-hash", XUATTR_OPERATOR_SSHA_HASH },
+	{ "imap-server", XUATTR_IMAP_SERVER },
+	{ "publish", XUATTR_PUBLISH },
+	{ "handle-pre", XUATTR_HANDLE_PRE },
+	{ "handle-want", XUATTR_HANDLE_WANT },
+	{ "handle-old", XUATTR_HANDLE_OLD },
+	{ "handle-forbidden", XUATTR_HANDLE_FORBIDDEN },
+	{ "email-address-old", XUATTR_EMAIL_ADDRESS_OLD },
+	{ NULL, 0 }
+};
+
+
+static uint32_t
+parse_attribute(char * attr)
+{
+    int i=0;
+    while(attrlist[i].name != NULL) {
+	if(strcmp(attrlist[i].name, attr) == 0) 
+	    return attrlist[i].value;
+
+	if(attrlist[i].value == atoi(attr))
+	    return attrlist[i].value;
+	i++;
+    }
+    return -1;
+}
 
-    d = ug1_open_using_config_file (CONFIG_FILE);
-    if (ERROR)
+static uint32_t *
+argv_to_attrs(int argc, char ** argv)
     {
-	err_print ();
-	return err_code ();
+    int i;
+    uint32_t * ret = malloc(argc * sizeof(uint32_t));
+
+    if(ret == NULL) {
+	fprintf(stderr, "Insufficient memory\n");
+	exit(1);
+    }
+
+    for (i = 0; i < argc; i++) {
+	ret[i] = parse_attribute(argv[i]);
+	if(ret[i] == -1) {
+	    fprintf(stderr, "Unknown attribute: %s.\n", argv[i]);
+	    exit(1);
+	}
+    }
+    return ret;
     }
 
-    if (strcmp (argv[1], "-ao") == 0)
+static uint32_t *
+argv_to_uids(int argc, char **argv)
     {
-	ugx_authorize_operator (d, argv[2], argv[3]);
+    uint32_t *ret;
+    int i;
+
+    ret = malloc(argc * sizeof(uint32_t));
+
+    if(ret == NULL) {
+	fprintf(stderr, "Out of memory\n");
+	exit(1);
+    }
+
+    for(i=0; i < argc; i++) {
+	ret[i] = atoi(argv[i]);
+    }
+
+    return ret;
+}
+
+static int help_cmd(int argc, char ** argv);
+static int user_data_multiuid_cmd(int argc, char ** argv);
+static int user_data_cmd(int argc, char ** argv);
+static int user_data_multiattr_cmd(int argc, char ** argv);
+static int user_lookup_batch_cmd(int argc, char **argv);
+static int change_operator_password_cmd(int argc, char **argv);
+static int add_user_data_cmd(int argc, char **argv);
+
+
+
+static int
+user_data_cmd(int argc, char **argv) {
+    char **v;
+    int i;
+
+    v = ug1_user_data (d, atoi (argv[1]), parse_attribute (argv[2]));
 	if (ERROR)
 	{
 	    err_print ();
 	    exit (err_code ());
 	}
-	argc -= 3;
-	argv += 3;
+	
+    for (i = 0; v[i] != NULL; i++)
+	printf ("%s\n", v[i]);
+    free (v);
+    return 0;
     }
 
-    if (strcmp (argv[1], "-au") == 0)
+static int
+user_data_multiuid_cmd(int argc, char **argv)
     {
-	ugx_authorize_user (d, argv[2], argv[3]);
+    char *** u;
+    uint32_t * uids;
+    int i,j;
+    int nuids = argc-2;
+
+    uids = argv_to_uids(nuids, argv+2);
+
+    u = ug1_user_data_multiuid(d, uids, nuids, parse_attribute(argv[1]));
 	if (ERROR)
 	{
 	    err_print ();
 	    exit (err_code ());
 	}
-	argc -= 3;
-	argv += 3;
+
+    for (i = 0; i < nuids; i++) {
+	for(j = 0 ; u[i][j] != NULL; j++) {
+	    if(j > 0)
+		printf("\t");
+	    
+	    printf("%s", u[i][j]);
+	    }
+	printf("\n");
     }
 
-    if (strcmp (argv[1], "ud") == 0)
+    return 0;
+}
+
+static int
+user_data_multiattr_cmd(int argc, char **argv)
     {
-	v = ug1_user_data (d, atoi (argv[2]), atoi (argv[3]));
+    char *** u;
+    uint32_t * attrs;
+    int i,j;
+    int nattrs = argc-2;
+    
+    attrs = argv_to_attrs(nattrs, argv+2);
+	
+    u = ug1_user_data_multiattr(d, atoi(argv[1]), attrs, nattrs);
 	if (ERROR)
 	{
 	    err_print ();
 	    exit (err_code ());
 	}
 	
-	for (i = 0; v[i] != NULL; i++)
-	    printf ("%s\n", v[i]);
-	free (v);
+    for (i = 0; i < nattrs; i++) {
+	for(j = 0 ; u[i][j] != NULL; j++) {
+	    if(j > 0)
+		printf("\t");
+
+	    printf("%s", u[i][j]);
+	}
+	printf("\n");
+    }
+
+    return 0;
     }
-    else if (strcmp (argv[1], "gd") == 0)
+
+
+static int
+group_data_cmd(int argc, char **argv) 
     {
-	v = ug1_group_data (d, atoi (argv[2]), atoi (argv[3]));
+    char **v;
+    int i;
+
+    v = ug1_group_data (d, atoi (argv[1]), atoi (argv[2]));
 	if (ERROR)
 	{
 	    err_print ();
@@ -75,9 +204,16 @@
 	for (i = 0; v[i] != NULL; i++)
 	    printf ("%s\n", v[i]);
 	free (v);
+
+    return 0;
     }
-    else if (strcmp (argv[1], "um") == 0)
+
+static int
+user_membership_cmd(int argc, char **argv) 
     {
+    uint32_t *r;
+    uint32_t *cp;
+
 	r = ug1_user_membership (d, atoi (argv[2]));
 	if (ERROR)
 	{
@@ -88,10 +224,20 @@
 	while (*cp != 0)
 	    printf ("%d\n", *cp++);
 	free (r);
+
+    return 0;
     }
-    else if (strcmp (argv[1], "gm") == 0)
+
+
+
+
+
+static int
+group_membership_cmd(int argc, char **argv)
     {
-	r = ug1_group_membership (d, atoi (argv[2]));
+    uint32_t *r, *cp;
+
+    r = ug1_group_membership (d, atoi (argv[1]));
 	if (ERROR)
 	{
 	    err_print ();
@@ -101,10 +247,15 @@
 	while (*cp != 0)
 	    printf ("%d\n", *cp++);
 	free (r);
+    return 0;
     }
-    else if (strcmp (argv[1], "ul") == 0)
+
+static int
+user_lookup(int argc, char **argv)
     {
-	r = ug1_user_lookup (d, atoi (argv[2]), argv[3]);
+    uint32_t *r, *cp;
+    
+    r = ug1_user_lookup (d, parse_attribute (argv[1]), argv[2]);
 	if (ERROR)
 	{
 	    err_print ();
@@ -114,10 +265,43 @@
 	while (*cp != 0)
 	    printf ("%d\n", *cp++);
 	free (r);
+    return 0;
+}
+
+static int
+user_lookup_batch_cmd(int argc, char **argv)
+{
+    uint32_t ** uids;
+    uint32_t *cp;
+    int i=0,j=0;
+    int nuids = argc-2;
+
+    uids = ug1_user_lookup_batch(d, parse_attribute (argv[1]), 
+				     nuids, argv+2);
+    if (ERROR)
+    {
+	err_print ();
+	exit (err_code ());
+    }
+    for(i=0; i < nuids ; i++)
+    {
+	cp = uids[j];
+	while(*cp != '\0') 
+	    printf("%d", *(cp++));
+	printf("\n");
+	j++; 
     }
-    else if (strcmp (argv[1], "gl") == 0)
+    free(uids);
+
+    return 0;
+    
+}
+static int
+group_lookup_cmd(int argc, char **argv)
     {
-	r = ug1_group_lookup (d, atoi (argv[2]), argv[3]);
+    uint32_t *r, *cp;
+
+    r = ug1_group_lookup (d, atoi (argv[1]), argv[2]);
 	if (ERROR)
 	{
 	    err_print ();
@@ -127,63 +311,85 @@
 	while (*cp != 0)
 	    printf ("%d\n", *cp++);
 	free (r);
+    return 0;
     }
-    else if (strcmp (argv[1], "aud") == 0)
+
+static int
+add_user_data_cmd(int argc, char **argv)
     {
-	ug1_add_user_data (d, atoi (argv[2]), atoi (argv[3]), argv[4]);
+    ug1_add_user_data (d, atoi (argv[1]), atoi (argv[2]), argv[3]);
 	if (ERROR)
 	{
 	    err_print ();
 	    exit (err_code ());
 	}
+    return 0;
     }
-    else if (strcmp (argv[1], "agd") == 0)
+
+static int
+add_group_data_cmd(int argc, char **argv)
     {
-	ug1_add_group_data (d, atoi (argv[2]), atoi (argv[3]), argv[4]);
+    ug1_add_group_data (d, atoi (argv[1]), atoi (argv[2]), argv[3]);
 	if (ERROR)
 	{
 	    err_print ();
 	    exit (err_code ());
 	}
+    return 0;
     }
-    else if (strcmp (argv[1], "rud") == 0)
+
+static int
+remove_user_data_cmd(int argc, char **argv)
     {
-	ug1_remove_user_data (d, atoi (argv[2]), atoi (argv[3]), argv[4]);
+    ug1_remove_user_data (d, atoi (argv[1]), atoi (argv[2]), argv[3]);
 	if (ERROR)
 	{
 	    err_print ();
 	    exit (err_code ());
 	}
+    return 0;
     }
-    else if (strcmp (argv[1], "rgd") == 0)
+
+static int
+remove_group_data_cmd(int argc, char **argv)
     {
-	ug1_remove_group_data (d, atoi (argv[2]), atoi (argv[3]), argv[4]);
+    ug1_remove_group_data (d, atoi (argv[1]), atoi (argv[2]), argv[3]);
 	if (ERROR)
 	{
 	    err_print ();
 	    exit (err_code ());
 	}
+    return 0;
     }
-    else if (strcmp (argv[1], "am") == 0)
+
+static int 
+add_membership_cmd(int argc, char **argv) 
     {
-	ug1_add_membership (d, atoi (argv[2]), atoi (argv[3]));
+    ug1_add_membership (d, atoi (argv[1]), atoi (argv[2]));
 	if (ERROR)
 	{
 	    err_print ();
 	    exit (err_code ());
 	}
+    return 0;
     }
-    else if (strcmp (argv[1], "rm") == 0)
+
+static int
+remove_membership_cmd(int argc, char **argv)
     {
-	ug1_remove_membership (d, atoi (argv[2]), atoi (argv[3]));
+    ug1_remove_membership (d, atoi (argv[1]), atoi (argv[2]));
 	if (ERROR)
 	{
 	    err_print ();
 	    exit (err_code ());
 	}
+    return 0;
     }
-    else if (strcmp (argv[1], "au") == 0)
+
+static
+int allocate_uid_cmd(int argc, char **argv)
     {
+    uint32_t uid;
 	uid = ug1_alloc_uid (d);
 	if (ERROR)
 	{
@@ -191,9 +397,13 @@
 	    exit (err_code ());
 	}
 	printf ("%d\n", uid);
+    return 0;
     }
-    else if (strcmp (argv[1], "ag") == 0)
+
+static int
+allocate_gid_cmd(int argc, char **argv)
     {
+    uint32_t gid;
 	gid = ug1_alloc_gid (d);
 	if (ERROR)
 	{
@@ -201,9 +411,13 @@
 	    exit (err_code ());
 	}
 	printf ("%d\n", gid);
+    return 0;
     }
-    else if (strcmp (argv[1], "mu") == 0)
+
+static int
+max_uid_cmd(int argc, char **argv)
     {
+    uint32_t uid;
 	uid = ug1_max_uid (d);
 	if (ERROR)
 	{
@@ -211,9 +425,13 @@
 	    exit (err_code ());
 	}
 	printf ("%d\n", uid);
+    return 0;
     }
-    else if (strcmp (argv[1], "mg") == 0)
+
+static int
+max_gid(int argc, char **argv)
     {
+    uint32_t gid;
 	gid = ug1_max_gid (d);
 	if (ERROR)
 	{
@@ -221,10 +439,14 @@
 	    exit (err_code ());
 	}
 	printf ("%d\n", gid);
+    return 0;
     }
-    else if (strcmp (argv[1], "users_with") == 0)
+static int
+users_with_cmd(int argc, char **argv)
     {
-	r = ug1_users_with_attribute (d, atoi (argv[2]));
+    uint32_t *r, *cp;
+
+    r = ug1_users_with_attribute (d, parse_attribute (argv[1]));
 	if (ERROR)
 	{
 	    err_print ();
@@ -234,10 +456,14 @@
 	while (*cp != 0)
 	    printf ("%d\n", *cp++);
 	free (r);
+    return 0;
     }
-    else if (strcmp (argv[1], "groups_with") == 0)
+
+static int
+groups_with_cmd(int argc, char **argv)
     {
-	r = ug1_groups_with_attribute (d, atoi (argv[2]));
+    uint32_t *r, *cp;
+    r = ug1_groups_with_attribute (d, atoi (argv[1]));
 	if (ERROR)
 	{
 	    err_print ();
@@ -247,9 +473,13 @@
 	while (*cp != 0)
 	    printf ("%d\n", *cp++);
 	free (r);
+    return 0;
     }
-    else if (strcmp (argv[1], "locker") == 0)
+
+static int
+locker_cmd(int argc, char **argv) 
     {
+    uint32_t client;
 	client = ug1_syslock_locker (d);
 	if (ERROR)
 	{
@@ -257,8 +487,11 @@
 	    exit (err_code ());
 	}
 	printf ("%d\n", client);
+    return 0;
     }
-    else if (strcmp (argv[1], "hard") == 0)
+
+static int
+hard_cmd(int argc, char **argv)
     {
 	ug1_syslock_hard (d);
 	if (ERROR)
@@ -275,8 +508,11 @@
 	    err_print ();
 	    exit (err_code ());
 	}
+    return 0;
     }
-    else if (strcmp (argv[1], "soft") == 0)
+
+static int
+soft_cmd(int argc, char **argv)
     {
 	ug1_syslock_soft (d);
 	if (ERROR)
@@ -293,8 +529,11 @@
 	    err_print ();
 	    exit (err_code ());
 	}
+    return 0;
     }
-    else if (strcmp (argv[1], "unlock") == 0)
+
+static int
+unlock_cmd(int argc, char **argv)
     {
 	ug1_syslock_unlock (d);
 	if (ERROR)
@@ -302,8 +541,11 @@
 	    err_print ();
 	    exit (err_code ());
 	}
+    return 0;
     }
-    else if (strcmp (argv[1], "shutdown") == 0)
+
+static int
+shutdown_cmd(int argc, char **argv)
     {
 	ug1_shutdown_server (d);
 	if (ERROR)
@@ -311,158 +553,301 @@
 	    err_print ();
 	    exit (err_code ());
 	}
+    return 0;
     }
-    else if (strcmp (argv[1], "change_password") == 0)
+
+static int
+change_password_cmd(int argc, char **argv)
     {
-	ugx_change_password (d, atoi (argv[2]), argv[3], argv[4]);
+    ugx_change_password (d, atoi (argv[1]), argv[2], argv[3]);
 	if (ERROR)
 	{
 	    err_print ();
 	    exit (err_code ());
 	}
+    return 0;
     }
-    else if (strcmp (argv[1], "set_new_password") == 0)
+
+static int
+set_new_password_cmd(int argc, char **argv)
     {
 	char *passwd;
 
-	passwd = ugx_set_new_password (d, atoi (argv[2]));
+    passwd = ugx_set_new_password (d, atoi (argv[1]));
 	if (ERROR)
 	{
 	    err_print ();
 	    exit (err_code ());
 	}
 	printf ("%s\n", passwd);
+    return 0;
     }
-    else if (strcmp (argv[1], "change_operator_password") == 0)
+
+static int
+change_operator_password_cmd(int argc, char **argv) 
     {
-	ugx_change_operator_password (d, atoi (argv[2]), argv[3], argv[4]);
+    ugx_change_operator_password (d, atoi (argv[1]), argv[2], argv[3]);
 	if (ERROR)
 	{
 	    err_print ();
 	    exit (err_code ());
 	}
+    return 0;
     }
-    else if (strcmp (argv[1], "set_new_operator_password") == 0)
+
+static int
+set_new_operator_password_cmd(int argc, char **argv)
     {
 	char *passwd;
 
-	passwd = ugx_set_new_operator_password (d, atoi (argv[2]));
+    passwd = ugx_set_new_operator_password (d, atoi (argv[1]));
 	if (ERROR)
 	{
 	    err_print ();
 	    exit (err_code ());
 	}
 	printf ("%s\n", passwd);
+    return 0;
     }
-    else if (strcmp (argv[1], "create_user") == 0)
+
+static int
+create_user_cmd(int argc, char **argv)
     {
-	uid = ugx_create_user (d, argv[2], argv[3], argv[4]);
+    uint32_t uid;
+    uid = ugx_create_user (d, argv[1], argv[2], argv[3]);
 	if (ERROR)
 	{
 	    err_print ();
 	    exit (err_code ());
 	}
 	printf ("New user has UID %d\n", uid);
+    return 0;
     }
-    else if (strcmp (argv[1], "ssha_hash") == 0)
+
+static int
+ssha_hash_cmd(int argc, char **argv)
     {
 	char hash[33];
 
 	ssha_hash (argv[2], hash);
 	printf ("%s\n", hash);
+    return 0;
     }
-    else if (strcmp (argv[1], "ssha_verify") == 0)
+
+static int
+ssha_verify_cmd(int argc, char **argv)
     {
-	if (ssha_verify (argv[2], argv[3]))
+    if (ssha_verify (argv[1], argv[2]))
 	    printf ("OK\n");
 	else
 	    printf ("No good!\n");
+    return 0;
     }
-    else if (strcmp (argv[1], "handle_available") == 0)
+
+static int
+handle_available_cmd(int argc, char **argv)
     {
 	int code;
 	char *msg;
 	
-	code = ugx_handle_available (d, argv[2], atoi (argv[3]), &msg);
+    code = ugx_handle_available (d, argv[1], atoi (argv[2]), &msg);
 	if (ERROR)
 	{
 	    err_print ();
 	    exit (err_code ());
 	}
 	printf ("%s\n", msg);
+    return 0;
     }
-    else if (strcmp (argv[1], "select_handle") == 0)
+
+static int
+select_handle_cmd(int argc, char **argv)
     {
-	ugx_select_handle (d, atoi (argv[2]), argv[3], argv[4]);
+    ugx_select_handle (d, atoi (argv[1]), argv[2], argv[3]);
 	if (ERROR)
 	{
 	    err_print ();
 	    exit (err_code ());
 	}
+    return 0;
     }
-    else if (strcmp (argv[1], "forbid_handle") == 0)
+
+static int
+forbid_handle_cmd(int argc, char **argv)
     {
-	ugx_forbid_handle (d, atoi (argv[2]), argv[3], argv[4]);
+    ugx_forbid_handle (d, atoi (argv[1]), argv[2], argv[3]);
 	if (ERROR)
 	{
 	    err_print ();
 	    exit (err_code ());
 	}
+    return 0;
     }
-    else if (strcmp (argv[1], "disable_user") == 0)
+
+static int
+disable_user_cmd(int argc, char **argv)
     {
-	ugx_disable_user (d, atoi (argv[2]), atoi (argv[3]), argv[4]);
+    ugx_disable_user (d, atoi (argv[1]), atoi (argv[2]), argv[3]);
 	if (ERROR)
 	{
 	    err_print ();
 	    exit (err_code ());
 	}
+    return 0;
     }
-    else if (strcmp (argv[1], "enable_user") == 0)
+
+static int
+enable_user_cmd(int argc, char **argv)
     {
-	ugx_enable_user (d, atoi (argv[2]), atoi (argv[3]), argv[4]);
+    ugx_enable_user (d, atoi (argv[1]), atoi (argv[2]), argv[3]);
 	if (ERROR)
 	{
 	    err_print ();
 	    exit (err_code ());
 	}
+    return 0;
     }
-    else if (strcmp (argv[1], "add_group_admins") == 0)
+
+static int
+add_group_admins_cmd(int argc, char **argv)
     {
-	ugx_add_group_admins (d, atoi (argv[2]), atoi (argv[3]));
+    ugx_add_group_admins (d, atoi (argv[1]), atoi (argv[2]));
 	if (ERROR)
 	{
 	    err_print ();
 	    exit (err_code ());
 	}
+    return 0;
     }
-    else if (strcmp (argv[1], "remove_group_admins") == 0)
+
+static int
+remove_group_admins_cmd(int argc, char **argv)
     {
-	ugx_remove_group_admins (d, atoi (argv[2]), atoi (argv[3]));
+    ugx_remove_group_admins (d, atoi (argv[1]), atoi (argv[2]));
 	if (ERROR)
 	{
 	    err_print ();
 	    exit (err_code ());
 	}
+    return 0;
     }
-    else if (strcmp (argv[1], "add_group_writers") == 0)
+
+static int 
+add_group_writers_cmd(int argc, char **argv)
     {
-	ugx_add_group_writers (d, atoi (argv[2]), atoi (argv[3]));
+    ugx_add_group_writers (d, atoi (argv[1]), atoi (argv[2]));
 	if (ERROR)
 	{
 	    err_print ();
 	    exit (err_code ());
 	}
+    return 0;
+}
+
+static int
+remove_group_writers_cmd(int argc, char **argv)
+{
+    ugx_remove_group_writers (d, atoi (argv[1]), atoi (argv[2]));
+    if (ERROR)
+    {
+	err_print ();
+	exit (err_code ());
+    }
+    return 0;
+}
+
+static SL_cmd cmds[] = {
+    {"user_data_multiuid", user_data_multiuid_cmd, "User data multiuid"},
+    {"user_data_multiattr", user_data_multiattr_cmd, "User data multiattr"},
+    {"user_data", user_data_multiuid_cmd, "User data"},
+    {"group_data", group_data_cmd, "Group data"},
+    {"user_membership", user_membership_cmd, "User membership"},
+    {"group_membership", group_membership_cmd, "Group membership"},
+    {"add_membership", add_membership_cmd, "Add membership"},
+    {"remove_membership", remove_membership_cmd, "Remove membership"},
+    {"user_lookup_batch", user_lookup_batch_cmd, "User data lookup batch"},
+    {"group_lookup", group_lookup_cmd, "Group lookup"},
+    {"change_operator_password", change_operator_password_cmd, "Change operator password"},
+    {"set_new_operator_password", set_new_operator_password_cmd, "Set new operator password"},
+    {"users_with", users_with_cmd, "Get users with certain attribute set"},
+    {"groups_with", groups_with_cmd, "Get groups with certain criteria"},
+    {"add_user_data", add_user_data_cmd, "Add data for user"},
+    {"help", help_cmd, "Show help"},
+    {NULL}
+
+}; 
+
+static int
+help_cmd (int argc, char **argv)
+{
+    SL_cmd *cmd;
+
+    for (cmd = cmds; cmd->name != NULL; ++cmd)
+        if (cmd->usage != NULL)
+            printf ("%-20s%s\n", cmd->name, cmd->usage);
+
+    return 0;
+}
+
+
+
+
+
+int
+main (int argc, char *argv[])
+{
+    uint32_t *r;
+    uint32_t *cp;
+    char **v;
+    int i;
+    uint32_t uid, gid;
+    uint32_t client;
+    int ret;
+
+    d = ug1_open_using_config_file (CONFIG_FILE);
+    if (ERROR)
+    {
+	err_print ();
+	return err_code ();
+    }
+
+    if(argc > 1) {
+	ret = sl_command(cmds, argc - 1 , argv + 1);
+	if (ret == SL_BADCOMMAND)
+	    printf ("%s: Unknown command\n", argv[1]); 
+    } else {
+	printf("fs - an arla tool for administrating AFS files.\n");
+	printf("Type \"help\" to get a list of commands.\n");
+	ret = sl_loop(cmds, "ugtest: ");  
+    }
+
+
+    exit(1);
+
+
+    if(strcmp (argv[1], "-ao") == 0) {
+	ugx_authorize_operator (d, argv[0], argv[1]);
+	if (ERROR)
+	{
+	    err_print ();
+	    exit (err_code ());
+	}
+	argc -= 3;
+	argv += 3;
     }
-    else if (strcmp (argv[1], "remove_group_writers") == 0)
+
+    if (strcmp (argv[1], "-au") == 0)
     {
-	ugx_remove_group_writers (d, atoi (argv[2]), atoi (argv[3]));
+	ugx_authorize_user (d, argv[2], argv[3]);
 	if (ERROR)
 	{
 	    err_print ();
 	    exit (err_code ());
 	}
+	argc -= 3;
+	argv += 3;
     }
+
     else
     {
 	printf ("Operation not known by this program!\n");
@@ -477,3 +862,4 @@
 
     return 0;
 }
+
